/**
 * @author David Laurell <david.laurell@gmail.com>
 * 
 * Copyright 2008 David Laurell
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package game;

import gameobjects.Player;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
/**
 * Stores the high score of the game.
 * 
 * @author David Laurell <david.laurell@gmail.com>
 */

public class Highscore {
	static final String FILENAME = "highscore.save";
	static final int SIZE = 10;
	private HighscoreItem[] highscore = new HighscoreItem[SIZE];

	public Highscore() {
		loadHighscore();
	}

	/**
	 * Adds a player highscore
	 * 
	 * @param Player
	 *            p the player to add
	 */
	public void add(Player player) {
		HighscoreItem hsi = new HighscoreItem(player.getName(),player.getPoints());
		for(int i=0;i<SIZE;i++) {
			if(highscore[i] == null || hsi.getScore() > highscore[i].getScore()) {
				for(int j=SIZE-1;j>i+1;j--) {
					highscore[j] = highscore[j-1];
				}
				highscore[i] = hsi;

				break;
			}
		}

		saveHighscore();
	}
	
	public boolean isHighscore(int score) {		
		for(int i=0;i<SIZE;i++) {
			HighscoreItem hs = highscore[i];
			
			if(hs == null)
				return true;
			
			if(score > hs.getScore())
				return true;
		}
		return false;
	}

	/**
	 * Loads highscore from the highscore file
	 */
	public void loadHighscore() {
		try {
			BufferedInputStream bis = new BufferedInputStream(new FileInputStream(FILENAME));
			ObjectInputStream ois = new ObjectInputStream(bis);
			
			highscore = (HighscoreItem[]) ois.readObject();
			ois.close();
		}
		catch(Exception e) {}
	}

	/**
	 * Saves highscore to the highscore file
	 */
	public void saveHighscore() {
		try {
			BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(FILENAME));
			ObjectOutputStream oos = new ObjectOutputStream(bos);

			oos.writeObject(highscore);
			oos.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Returns a string containing a readable list of the highscores
	 * 
	 * @return String the higshcore list
	 */
	public HighscoreItem[] getList() {
		return highscore;
	}
}
