/**
 * Basic functions and variables for an object in the game.
 * 
 * @author David Laurell <david.laurell@gmail.com>
 * 
 * Copyright 2008 David Laurell
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package gameobjects;
/**
 * @author David Laurell <david.laurell@gmail.com>
 */
import org.newdawn.slick.Graphics;
import org.newdawn.slick.svg.SimpleDiagramRenderer;

public abstract class GameObject {
	protected float x=0,y=0;
	protected float hspeed=0, vspeed=0;
	protected boolean moving = false;
	protected boolean stopturn = false;
	protected int radius;
	protected float direction, rspeed;
	public int size;

	protected SimpleDiagramRenderer sprite;

	public void draw(Graphics g) {
		g.translate(x, y);
		g.rotate(0,0,direction+90);
		sprite.render(g);
		g.resetTransform();
	}
	public float getDirection() {
		return direction;
	}
	public float getX() {
		return x;
	}
	public float getY() {
		return y;
	}
	public int getRadius() {
		return radius;
	}
	public void setDirection(float angle) {
		direction = angle;
		
		if(direction > 360)
			direction -= 360;
		else if(direction < 0)
			direction += 360;
	}
	
	public boolean isOutside() {
		return (x > 800 || x < 0 || y > 600 || y < 0);
	}
	
	public void setSpeed(float speed) {
		/* converts how much to move diagonally to x and y values */
		hspeed += (float)(Math.cos(Math.toRadians((double)direction)) * speed);
		vspeed += (float)(Math.sin(Math.toRadians((double)direction)) * speed);
	}
	
	protected void moveObject(long delta) {
		x += (hspeed * delta) / 1000;
		y += (vspeed * delta) / 1000;
		
		/* rotate object */
		direction += delta * rspeed;
		
		if(direction < 0)
			direction += 360;
		else if(direction >= 360)
			direction -= 360;
		
		if(stopturn)
			rspeed = 0;
	}
	
	public void move(long delta)
	{	
		moveObject(delta);
		
		/* Move objects outside room to the other side */		
		if(x > 800)
			x -= 800;
		else if(x < 0)		
			x += 800;
		
		if(y > 600)
			y -= 600;
		else if(y < 0)
			y += 600;
	}
	
	public boolean isCollision(GameObject other)
	{
		double distx = x + size - other.getX() - other.size;
		double disty = y + size - other.getY() - other.size;
		
		return Math.sqrt(Math.pow(distx, 2) + Math.pow(disty, 2)) <= this.getRadius() + other.getRadius();
	}
}
