/**
 * The player class handles all player specific method such as how to move and shoot.
 * 
 * @author David Laurell <david.laurell@gmail.com>
 * 
 * Copyright 2008 David Laurell
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package gameobjects;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.svg.*;
import game.Alarm;

public class Player extends GameObject {
	private static float ROTATE_SPEED = 0.25f;
	private int shootSpeed = 500;
	private boolean isShooting = false;
	private Alarm dontshoot, invtimer, deadtimer;
	private boolean dead, invulnerable;
	private int lives=3, points;
	private String name;
	
	private SimpleDiagramRenderer imgShield;
		
	public Player(int x, int y, SimpleDiagramRenderer sprite, SimpleDiagramRenderer shieldsprite)
	{	
		this.x = x;
		this.y = y;
		this.sprite = sprite;
		this.radius = 16;
		this.dontshoot = new Alarm();
		this.invtimer = new Alarm();
		this.deadtimer = new Alarm();
		this.dead = false;
		this.stopturn = true;
		this.imgShield = shieldsprite;
		this.name = "Player";
	}
	
	public int getPoints() {
		return points;
	}
	
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public int getLives() {
		return lives;
	}
	
	public void draw(Graphics g) {
		if(!isDead()) {
			g.translate(x, y);
			g.rotate(0,0,direction+90);
			if(isInvulnerable())
				imgShield.render(g);
			else
				sprite.render(g);
			
			sprite.render(g);
			g.resetTransform();
		}
	}
	
	public void setInvulnerable(int time) {
		invulnerable = true;
		invtimer.start(time);
	}
	
	public boolean isInvulnerable() {
		return invulnerable;
	}
	
	public boolean isDead() {
		return dead;
	}
	
	public void setDead() {
		lives--;
		deadtimer.start(1000);
		dead = true;
	}
	
	public void addPoints(int amount) {
		points += amount;
	}
	
	public void turnRight() {
		rspeed = +ROTATE_SPEED;
	}

	public void turnLeft() {
		rspeed = -ROTATE_SPEED;
	}
	
	public void shoot() {
		if(!isShooting && !dontshoot.isAlarm()) {
			isShooting = true;
		}
	}
	
	public void update(int delta) {
		if(!invtimer.isAlarm() && invulnerable)
			invulnerable = false;

		if(!deadtimer.isAlarm() && dead && lives > 0) {
			dead = false;
			respawn();
			setInvulnerable(1500);
		}
	}
	
	public boolean makeShot() {
		if(isShooting && !dontshoot.isAlarm()) {
			isShooting = false;
			dontshoot.start(shootSpeed);
			return true;
		}
		else
			return false;
	}
	
	public void respawn() {
		x = 400;
		y = 300;
		hspeed = 0;
		vspeed = 0;
	}
}
